/*
 * This file is part of FGGL.
 *
 * FGGL is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 *
 * FGGL is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along with FGGL.
 * If not, see <https://www.gnu.org/licenses/>.
 */

//
// stateless component factories that are probably safe
//

#include "fggl/entity/module.hpp"
#include "fggl/math/types.hpp"
#include "fggl/scenes/game.hpp"

namespace fggl::entity {

	void make_transform(const entity::ComponentSpec &spec, EntityManager &manager, const entity::EntityID entity, modules::Services &/*svc*/) {
		auto &transform = manager.add<math::Transform>(entity);

		//FIXME won't work for patching!
		transform.origin(spec.get<math::vec3>("origin", math::VEC3_ZERO));
		transform.euler(spec.get<math::vec3>("rotation", math::VEC3_ZERO));
		transform.scale(spec.get<math::vec3>("scale", math::VEC3_ONES));

		debug::trace("created transform for entity {}", (uint64_t) entity);
	}

	void install_component_factories(entity::EntityFactory *factory) {
		factory->bind(math::Transform::guid, make_transform);
	}

	bool ECS::factory(modules::ModuleService service, modules::Services &services) {
		if (service == EntityFactory::service) {
			auto *factory = services.create<EntityFactory>(services);
			install_component_factories(factory);

			// we are responsible for prefabs...
			auto *assetLoader = services.get<assets::Loader>();
			assetLoader->setFactory(PROTOTYPE_ASSET, [factory](assets::Loader* loader, const assets::AssetGUID &a, assets::AssetData b, void* ptr) {
				return load_prototype(loader, factory, a, b);
			}, assets::LoadType::PATH);
			assetLoader->setFactory(SCENE, load_scene, assets::LoadType::PATH);

			return true;
		}
		return false;
	}
}